/******************************************************************/
/* filename: cfpe.c */
/* version : 1.03 */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "processConfig.h"
#include "debug.h"

/******************************************************************/
/* MSAP pipe declaration */
int MSAP_datareq_pipe[2];
int MSAP_dataind_pipe[2];

/******************************************************************/
/* External declaration of pipes [defined in cp.c] */
extern int exit_mc_st_pipe[2]; /* We use only the read pipe [0]! */
extern int exit_mc_rt_pipe[2]; /* We use only the read pipe [0]! */
extern int exit_cf_st_pipe[2]; /* We use only the read pipe [0]! */
extern int exit_cf_rt_pipe[2]; /* We use only the read pipe [0]! */

/******************************************************************/
/* external declaration of processes */
extern int cf_pe(); /* See cf_pe.c */
extern int mcpe();  /* See mcpe.c */

/******************************************************************/
/* function prototying (forward declaration) */
int cfpe(char *config);

/******************************************************************/

int cfpe(char *config)
{
 int PId;
 struct ConfPartnersStruc pp;
 if (loadConfig(&pp,config)!=0) 
 {
  Debug(Dbg,"Config file erroneous!");
  return(-1);
 }	

 /* CFPE_init means creation the MSAP pipes for cf_pe() and mcpe() */
 if (pipe(MSAP_datareq_pipe)!=0) 
 {
  Error(Cfpe,"[cfpe:main()] Error creating MSAP_datareq_pipe\n");
  return(-2);
 }
 if (pipe(MSAP_dataind_pipe)!=0)
 {
  Error(Cfpe,"cfpe.c: Creating MSAP_dataind_pipe\n");
  /* Note that MSAP_datareq_pipe isn't closed! */
  close(MSAP_datareq_pipe[0]);
  close(MSAP_datareq_pipe[1]);
  return (-3);
 }

 if ((PId = fork()) == -1)
 {
  Error(Cfpe,"cfpe.c: Executing fork()\n");
  return(-4);
 }

 if (PId)
 {
  int rvalue;

  /* close the pipes which the cf_pe() does not use! */
  close(MSAP_datareq_pipe[0]);
  close(MSAP_dataind_pipe[1]);

  /* Startup the user() */
  rvalue = cf_pe(&pp);
  Debug(Dbg,"cfpe.c: Exit cf_pe() detected...");
  
  /* close other pipes */
  close(MSAP_datareq_pipe[1]);
  close(MSAP_dataind_pipe[0]);

  return (rvalue<0 ? rvalue-4 : rvalue);
 }
 else
 {
  int rvalue;

  /* close the pipes which the mcpe process does not use! */
  close(MSAP_datareq_pipe[1]);
  close(MSAP_dataind_pipe[0]);

  /* startup mcpe() */
  rvalue = mcpe(&pp);
  Debug(Dbg,"cfpe.c: Exit mcpe() detected...");

  /* close the other pipes */
  close(MSAP_datareq_pipe[0]);
  close(MSAP_dataind_pipe[1]);

  return (rvalue<0 ? rvalue-4 : rvalue);
 }
}

