/******************************************************************/
/* filename: mc_pe.c  */
/* version : 1.00 */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "processConfig.h"
#include <stdlib.h>
#include <signal.h>
#include "debug.h"

/******************************************************************/
/* external declaration of processes */
extern int mc_st(); /* See mc_st.c */
extern int mc_rt(); /* See mc_rt.c */

/******************************************************************/
/* Declaration of local variables */
static int PId=0;				/* process identifier */ 

/******************************************************************/
 static void terminate(int unused)
 {
  if (PId > 0) 
  {
   kill(PId, SIGTERM);
   Debug(Dbg,"mc_pe.c: terminated %d\n", PId);
   PId = 0;
  }
  exit(0);
 }
/******************************************************************/

int mc_pe(struct ConfPartnersStruc *pp)
{
 /* Set desired termination signals */
 signal(SIGHUP, terminate);
 signal(SIGTERM, terminate);
 signal(SIGQUIT, terminate);
 signal(SIGKILL, terminate);
 signal(SIGINT, terminate);
 signal(SIGCHLD, terminate);
 
if ((PId = fork()) == -1)
 {
  Error(Dbg,"mc_pe.c: Error occured during fork()\n");
  return(-1);
 }

 if (PId)
 {
  int rvalue;

  /* Startup the mc_st() */
  rvalue = mc_st(pp);
  terminate(0);
  Debug(Dbg,"mc_pe.c: Exit Multicast Send Task detected...");
  return rvalue;
 }
 else
 {
  int rvalue;

  /* startup mc_rt() */
  rvalue = mc_rt(pp);
  terminate(0);
  Debug(Dbg,"mc_pe.c: Exit Multicast Receive Task detected...");
  return rvalue;
 }
}

