/******************************************************************/
/* filename: mc_rt.c */
/* version : 1.00 */
/* author  : Jan Feenstra */
/******************************************************************/

#include "cp.h"
#include "cfpe.h"
#include "processConfig.h"
#include "debug.h"

/******************************************************************/
/* External declaration of pipes */
extern int MSAP_dataind_pipe[2];

/******************************************************************/
/* function prototying (forward declaration) */
int mc_rt(struct ConfPartnersStruc *p);

/******************************************************************/

int mc_rt( struct ConfPartnersStruc *p)
{
 int i;
 struct dataind_mdu_Type mdu;
 struct sockaddr_in source;
 char buf1[20];
 char buf2[20];
 fd_set read_mask;
 
 Debug(Mc_rt,"mc_rt.c: Entering Multicast Receive Task Loop.");

 while (1) 
 {
  FD_ZERO(&read_mask);
  FD_SET(p->user.usap, &read_mask);
  
  select(FD_SETSIZE, &read_mask, (fd_set *) 0, (fd_set *) 0, 0);

  if (FD_ISSET(p->user.usap, &read_mask))
  {
   USAP_dataind_proc(p->user.usap, \
					&source, \
					&mdu.data, \
                    MAXPDUSIZE,
					&mdu.size);
					
   for (i=0; i<p->size; i++)
   {
    strcpy((char *)&buf1,inet_ntoa(p->Partner[i].socketaddress.sin_addr));
    strcpy((char *)&buf2,inet_ntoa(source.sin_addr));

    if ( 
	   (strcmp(buf1,buf2) == 0) \
       && (p->Partner[i].socketaddress.sin_port == source.sin_port) \
      ) 
    {
	 mdu.srcAddress=p->Partner[i].msapid;
	 
	switch (mdu.data[0])
	{
	 case '\1':
	 {
 	  Debug(Mc_rt,"mc_rt.c: reading JOIN-pdu (%d bytes) from user %d", \
	   mdu.size, mdu.srcAddress);
	  break;
	 }
	 case '\2':
	 {
 	  Debug(Mc_rt,"mc_rt.c: reading LEAVE-pdu (%d bytes) from user %d", \
	   mdu.size, mdu.srcAddress);
	  break;
	 }
	 case '\3':
	 {
 	  Debug(Mc_rt,"mc_rt.c: reading ANSWER-pdu (%d bytes) from user %d", \
	   mdu.size, mdu.srcAddress);
	  break;
	 }
	 case '\4':
	 {
 	  Debug(Mc_rt,"mc_rt.c: reading DATA-pdu (%d bytes) from user %d", \
	   mdu.size, mdu.srcAddress);
	  break;
	 }
	 default:
	 {
 	  Debug(Mc_rt,"mc_rt.c: reading Unknown pdu (%d bytes) from user %d", \
	   mdu.size, mdu.srcAddress);
	  break;
	 }
	 
	}
	
    write(MSAP_dataind_pipe[1], (char *)&mdu, sizeof(struct dataind_mdu_Type));
  	break;
    }
   }
  }
 }
}
/******************************************************************/

